@@ -78,6 +78,10 @@ const (
 	trackerLength    = len(uuid.UUID{})
 	protocolICMP     = 1
 	protocolIPv6ICMP = 58
+
+	networkIP   = "ip"
+	networkIPv4 = "ip4"
+	networkIPv6 = "ip6"
 )
 
 var (
@@ -98,6 +102,7 @@ func New(addr string) *Pinger {
 		Count:      -1,
 		Interval:   time.Second,
 		RecordRtts: true,
+		RecordTTLs: true,
 		Size:       timeSliceLength + trackerLength,
 		Timeout:    time.Duration(math.MaxInt64),
 
@@ -107,10 +112,11 @@ func New(addr string) *Pinger {
 		trackerUUIDs:      []uuid.UUID{firstUUID},
 		ipaddr:            nil,
 		ipv4:              false,
-		network:           "ip",
+		network:           networkIP,
 		protocol:          "udp",
 		awaitingSequences: firstSequence,
 		TTL:               64,
+		tclass:            192, // CS6 (network control)
 		logger:            StdLogger{Logger: log.New(log.Writer(), log.Prefix(), log.Flags())},
 	}
 }
@@ -130,6 +136,9 @@ type Pinger struct {
 	// packets have been received.
 	Timeout time.Duration
 
+	// ResolveTimeout specifies a timeout to resolve an IP address or domain name
+	ResolveTimeout time.Duration
+
 	// Count tells pinger to stop after sending (and receiving) Count echo
 	// packets. If this option is not specified, pinger will operate until
 	// interrupted.
@@ -152,16 +161,23 @@ type Pinger struct {
 	maxRtt    time.Duration
 	avgRtt    time.Duration
 	stdDevRtt time.Duration
-	stddevm2  time.Duration
+	stddevm2  float64
 	statsMu   sync.RWMutex
 
 	// If true, keep a record of rtts of all received packets.
 	// Set to false to avoid memory bloat for long running pings.
 	RecordRtts bool
 
+	// If true, keep a record of TTLs of all received packets.
+	// Set to false to avoid memory bloat for long running pings.
+	RecordTTLs bool
+
 	// rtts is all of the Rtts
 	rtts []time.Duration
 
+	// ttls is all of the TTLs
+	ttls []uint8
+
 	// OnSetup is called when Pinger has finished setting up the listening socket
 	OnSetup func()
 
@@ -192,6 +208,9 @@ type Pinger struct {
 	// Source is the source IP address
 	Source string
 
+	// Interface used to send/recv ICMP messages
+	InterfaceName string
+
 	// Channel and mutex used to communicate when the Pinger should stop between goroutines.
 	done chan interface{}
 	lock sync.Mutex
@@ -221,12 +240,16 @@ type Pinger struct {
 	logger Logger
 
 	TTL int
+
+	// tclass defines the traffic class (ToS for IPv4) set on outgoing icmp packets
+	tclass uint8
 }
 
 type packet struct {
 	bytes  []byte
 	nbytes int
 	ttl    int
+	addr   net.Addr
 }
 
 // Packet represents a received and processed ICMP echo packet.
@@ -277,6 +300,9 @@ type Statistics struct {
 	// Rtts is all of the round-trip times sent via this pinger.
 	Rtts []time.Duration
 
+	// TTLs is all of the TTLs received via this pinger.
+	TTLs []uint8
+
 	// MinRtt is the minimum round-trip time sent via this pinger.
 	MinRtt time.Duration
 
@@ -300,6 +326,10 @@ func (p *Pinger) updateStatistics(pkt *Packet) {
 		p.rtts = append(p.rtts, pkt.Rtt)
 	}
 
+	if p.RecordTTLs {
+		p.ttls = append(p.ttls, uint8(pkt.TTL))
+	}
+
 	if p.PacketsRecv == 1 || pkt.Rtt < p.minRtt {
 		p.minRtt = pkt.Rtt
 	}
@@ -314,9 +344,9 @@ func (p *Pinger) updateStatistics(pkt *Packet) {
 	delta := pkt.Rtt - p.avgRtt
 	p.avgRtt += delta / pktCount
 	delta2 := pkt.Rtt - p.avgRtt
-	p.stddevm2 += delta * delta2
+	p.stddevm2 += float64(delta) * float64(delta2)
 
-	p.stdDevRtt = time.Duration(math.Sqrt(float64(p.stddevm2 / pktCount)))
+	p.stdDevRtt = time.Duration(math.Sqrt(p.stddevm2 / float64(pktCount)))
 }
 
 // SetIPAddr sets the ip address of the target host.
@@ -337,9 +367,42 @@ func (p *Pinger) Resolve() error {
 	if len(p.addr) == 0 {
 		return errors.New("addr cannot be empty")
 	}
-	ipaddr, err := net.ResolveIPAddr(p.network, p.addr)
-	if err != nil {
-		return err
+	var (
+		ipaddr *net.IPAddr
+		err    error
+	)
+	if p.ResolveTimeout > time.Duration(0) {
+		var (
+			ctx = context.Background()
+			ips []net.IP
+		)
+		ctx, cancel := context.WithTimeout(ctx, p.ResolveTimeout)
+		defer cancel()
+		ips, err = net.DefaultResolver.LookupIP(ctx, p.network, p.addr)
+		if err != nil {
+			return err
+		}
+		if len(ips) == 0 {
+			return fmt.Errorf("lookup %s failed: no addresses found", p.addr)
+		}
+		ipaddr = &net.IPAddr{IP: ips[0]}
+		for _, ip := range ips {
+			if p.network == networkIPv6 {
+				if ip.To4() == nil && ip.To16() != nil {
+					ipaddr = &net.IPAddr{IP: ip}
+					break
+				}
+				continue
+			}
+			if ip.To4() != nil {
+				ipaddr = &net.IPAddr{IP: ip}
+			}
+		}
+	} else {
+		ipaddr, err = net.ResolveIPAddr(p.network, p.addr)
+		if err != nil {
+			return err
+		}
 	}
 
 	p.ipv4 = isIPv4(ipaddr.IP)
@@ -373,12 +436,12 @@ func (p *Pinger) Addr() string {
 // * "ip6" will select IPv6.
 func (p *Pinger) SetNetwork(n string) {
 	switch n {
-	case "ip4":
-		p.network = "ip4"
-	case "ip6":
-		p.network = "ip6"
+	case networkIPv4:
+		p.network = networkIPv4
+	case networkIPv6:
+		p.network = networkIPv6
 	default:
-		p.network = "ip"
+		p.network = networkIP
 	}
 }
 
@@ -429,6 +492,18 @@ func (p *Pinger) SetDoNotFragment(df bool) {
 	p.df = df
 }
 
+// SetTrafficClass sets the traffic class (type-of-service field for IPv4) field
+// value for future outgoing packets.
+func (p *Pinger) SetTrafficClass(tc uint8) {
+	p.tclass = tc
+}
+
+// TrafficClass returns the traffic class field (type-of-service field for IPv4)
+// value for outgoing packets.
+func (p *Pinger) TrafficClass() uint8 {
+	return p.tclass
+}
+
 // Run runs the pinger. This is a blocking function that will exit when it's
 // done. If Count or Interval are not specified, it will run continuously until
 // it is interrupted.
@@ -468,7 +543,20 @@ func (p *Pinger) RunWithContext(ctx context.Context) error {
 		}
 	}
 
+	if p.tclass != 0 {
+		if err := conn.SetTrafficClass(p.tclass); err != nil {
+			return fmt.Errorf("error setting traffic class: %v", err)
+		}
+	}
+
 	conn.SetTTL(p.TTL)
+	if p.InterfaceName != "" {
+		iface, err := net.InterfaceByName(p.InterfaceName)
+		if err != nil {
+			return err
+		}
+		conn.SetIfIndex(iface.Index)
+	}
 	return p.run(ctx, conn)
 }
 
@@ -602,6 +690,7 @@ func (p *Pinger) Statistics() *Statistics {
 		PacketsRecvDuplicates: p.PacketsRecvDuplicates,
 		PacketLoss:            loss,
 		Rtts:                  p.rtts,
+		TTLs:                  p.ttls,
 		Addr:                  p.addr,
 		IPAddr:                p.ipaddr,
 		MaxRtt:                p.maxRtt,
@@ -653,9 +742,7 @@ func (p *Pinger) recvICMP(
 			if err := conn.SetReadDeadline(time.Now().Add(delay)); err != nil {
 				return err
 			}
-			var n, ttl int
-			var err error
-			n, ttl, _, err = conn.ReadFrom(bytes)
+			n, ttl, addr, err := conn.ReadFrom(bytes)
 			if err != nil {
 				if p.OnRecvError != nil {
 					p.OnRecvError(err)
@@ -673,7 +760,7 @@ func (p *Pinger) recvICMP(
 			select {
 			case <-p.done:
 				return nil
-			case recv <- &packet{bytes: bytes, nbytes: n, ttl: ttl}:
+			case recv <- &packet{bytes: bytes, nbytes: n, ttl: ttl, addr: addr}:
 			}
 		}
 	}
@@ -722,10 +809,24 @@ func (p *Pinger) processPacket(recv *packet) error {
 		return nil
 	}
 
+	// If initial ip is a broadcast ip, ping responses will come from machines' in the
+	// subnet, thus ip will differ. Below gets real ip from received package.
+	var realIP *net.IPAddr
+
+	switch v := recv.addr.(type) {
+	case *net.IPAddr: // For ICMP
+		realIP = v
+	case *net.UDPAddr:
+		realIP = &net.IPAddr{IP: v.IP}
+	default:
+		p.logger.Infof("received address: %s it neither an Ip address (ICMP) nor UDP address, shouldn't happen. using initial address", recv.addr)
+		realIP = p.ipaddr
+	}
+
 	inPkt := &Packet{
 		Nbytes: recv.nbytes,
-		IPAddr: p.ipaddr,
-		Addr:   p.addr,
+		IPAddr: realIP,
+		Addr:   realIP.String(),
 		TTL:    recv.ttl,
 		ID:     p.id,
 	}
@@ -807,6 +908,14 @@ func (p *Pinger) sendICMP(conn packetConn) error {
 
 	for {
 		if _, err := conn.WriteTo(msgBytes, dst); err != nil {
+			// Try to set broadcast flag
+			if errors.Is(err, syscall.EACCES) && runtime.GOOS == "linux" {
+				if e := conn.SetBroadcastFlag(); e != nil {
+					p.logger.Warnf("had EACCES syscall error, check your local firewall")
+				}
+				p.logger.Infof("Pinging a broadcast address")
+				continue
+			}
 			if p.OnSendError != nil {
 				outPkt := &Packet{
 					Nbytes: len(msgBytes),
